/*
 * engine_client_c.h
 *
 * $Copyright(c) 2002-2012 Progress Software Corporation and/or its subsidiaries and affiliates. All rights reserved.$
 * $Copyright (c) 2013-2017, 2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 *
 * $Id: engine_client_c.h 349681 2019-04-08 16:03:47Z matj $
 */

/** @file
 * This is the header file for the C engine_client API; as of 10.1 this is an internal-only API; 
 * customers should use the C++ API instead.
 */
#ifndef ENGINE_CLIENT_C_H
#define ENGINE_CLIENT_C_H


#include <AP_Types.h>
#include <AP_Platform.h>

#ifndef ENGINE_CLIENT_CPP_HPP
#ifdef __GNUC__
#warning The engine_client_c API is no longer a supported public API. Please use the C++ API located in engine_client_cpp.hpp instead
#else
#pragma message("engine_client_c.h(0) Warning: The engine_client_c API is no longer a supported public API. Please use the C++ API located in engine_client_cpp.hpp instead")
#endif
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


	struct AP_Event_API;
	struct AP_EventSupplier_API;
	struct AP_EventConsumer_API;
	struct AP_DisconnectableEventConsumer_API;
	struct AP_MonitorScript_API;
	struct AP_EngineStatus_API;
	struct AP_NameInfo_API;
	struct AP_NamedMonitorInfo_API;
	struct AP_NamedJavaApplicationInfo_API;
	struct AP_NamedEventTypeInfo_API;
	struct AP_NamedTimerInfo_API;
	struct AP_NamedContextInfo_API;
	struct AP_NamedAggregateInfo_API;
	struct AP_NamedReceiverInfo_API;
	struct AP_NamedPluginReceiverInfo_API;
	struct AP_EngineInfo_API;
	struct AP_EngineManagement_API;

#ifdef __cplusplus
	struct AP_Event_Stubs;
	struct AP_EventSupplier_Stubs;
	struct AP_EventConsumer_Stubs;
	struct AP_DisconnectableEventConsumer_Stubs;
	struct AP_MonitorScript_Stubs;
	struct AP_EngineStatus_Stubs;
	struct AP_NameInfo_Stubs;
	struct AP_NamedMonitorInfo_Stubs;
	struct AP_NamedJavaApplicationInfo_Stubs;
	struct AP_NamedEventTypeInfo_Stubs;
	struct AP_NamedTimerInfo_Stubs;
	struct AP_NamedContextInfo_Stubs;
	struct AP_NamedAggregateInfo_Stubs;
	struct AP_NamedReceiverInfo_Stubs;
	struct AP_NamedPluginReceiverInfo_Stubs;
	struct AP_EngineInfo_Stubs;
	struct AP_EngineManagement_Stubs;

	/**
	* An Event represents an event instance.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_Event_Stubs AP_Event;

	/**
	* An AP_EventSupplier represents the resources created by the Engine
	* to service a connection to an external sink of events. It
	* filters the event output of the Engine by delivering only the
	* events emitted on a particular set of channels. An
	* AP_EventSupplier allows to pass events to an AP_EventConsumer. 
	* AP_EventSupplier instances should be freed using the 
	* AP_DeleteEventSupplier function. 
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_EventSupplier_Stubs AP_EventSupplier;

	/**
	* An AP_EventConsumer allows to connect to the Engine through an 
	* AP_EventSupplier and register to receive events. In order to
	* receive events from the Engine, a developer must provide
	* sendEvents callback function. This function is
	* called when events are emitted from the
	* Engine.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_EventConsumer_Stubs AP_EventConsumer;

	/**
	* An AP_DisconnectableEventConsumer allows to connect to the Engine through an 
	* AP_EventSupplier and register to receive events. In order to
	* receive events from the Engine, a developer must provide
	* sendEvents callback function to s_EventConsumer member of 
	* the AP_DisconnectableEventConsumer. This function is
	* called when events are emitted from the
	* Engine.
	* The structure provides disconnect callback which is called 
	* to inform the consumer that it is not going 
	* to be sent any more events.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_DisconnectableEventConsumer_Stubs AP_DisconnectableEventConsumer;

	/**
	* A AP_MonitorScript allows MonitorScript code
	* fragment, containing package, event and monitor definitions to
	* be injected into an Engine.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_MonitorScript_Stubs AP_MonitorScript;

	/**
	* AP_EngineStatus allows access to the operational status of the Engine.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_EngineStatus_Stubs AP_EngineStatus;

	/**
	* Allows access to a named object (i.e. event type, container type
	* or monitor) returned by an engine inspection. Returned by functions
	* of the AP_EngineInfo structure. Strings returned by methods of this class
	* are valid until the AP_EngineInfo structure is deleted.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_NameInfo_Stubs AP_NameInfo;

	/**
	* Information about a monitor returned by an engine inspection.
	* Returned by the getMonitors function of the AP_EngineInfo structure.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_NamedMonitorInfo_Stubs AP_NamedMonitorInfo;

	/**
	* Information about a java application returned by an engine inspection.
	* Returned by the getJavaApplications method of the AP_EngineInfo structure.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_NamedJavaApplicationInfo_Stubs AP_NamedJavaApplicationInfo;

	/**
	* Information about a event type returned by an engine inspection.
	* Returned by the getEventTypes method of the AP_EngineInfo structure.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_NamedEventTypeInfo_Stubs AP_NamedEventTypeInfo;

	/**
	* Information about a timer type returned by an engine inspection.
	* Returned by the getTimers method of the AP_EngineInfo structure.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_NamedTimerInfo_Stubs AP_NamedTimerInfo;

	/**
	* Information about a context returned by an engine inspection.
	* Returned by the getContexts method of the AP_EngineInfo structure.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_NamedContextInfo_Stubs AP_NamedContextInfo;

	/**
	* Information about an aggregate function returned by an engine
	* inspection.
	* Returned by the getAggregates method of the AP_EngineInfo structure.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_NamedAggregateInfo_Stubs AP_NamedAggregateInfo;

	/**
	* Information about a receiver returned by an engine
	* inspection.
	* Returned by the getReceivers method of the AP_EngineInfo structure.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_NamedReceiverInfo_Stubs AP_NamedReceiverInfo;

	/**
	* Information about a plugin receiver returned by an engine
	* inspection.
	* Returned by the getPluginReceivers method of the AP_EngineInfo structure.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_NamedPluginReceiverInfo_Stubs AP_NamedPluginReceiverInfo;

	/**
	* Information about the monitors and types currently in an engine.
	* Instances of AP_EngineInfo structure are returned by the inspectEngine 
	* method of the AP_EngineManagement structure and can be deleted with the
	* AP_DeleteEngineInfo function. 
	*
	* When an AP_EngineInfo instance is deleted, everything returned by the 
	* getMonitors, getEventTypes, etc methods is deleted.
	* This includes the arrays themselves, the classes pointed to by
	* the arrays and any strings returned by those classes. This means
	* that AP_DeleteEngineInfo is the only cleanup method that needs to be called
	* after an engine inspection.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_EngineInfo_Stubs AP_EngineInfo;

	/**
	* The AP_EngineManagement structure allows access to the Engine. 
	* The static functions supplied below
	* must be used to create an instance of this structure.
	*
	* Because an AP_EngineManagement instance is also an AP_EventConsumer,
	* it can have events sent to it (and to the Engine) through its
	* sendEvents() function.
	* @deprecated The C++ API should be used in preference
	*/
	typedef struct AP_EngineManagement_Stubs AP_EngineManagement;

#else
	typedef struct AP_Event_API AP_Event;
	typedef struct AP_EventSupplier_API AP_EventSupplier;
	typedef struct AP_EventConsumer_API AP_EventConsumer;
	typedef struct AP_DisconnectableEventConsumer_API AP_DisconnectableEventConsumer;
	typedef struct AP_MonitorScript_API AP_MonitorScript;
	typedef struct AP_EngineStatus_API AP_EngineStatus;
	typedef struct AP_NameInfo_API AP_NameInfo;
	typedef struct AP_NamedMonitorInfo_API AP_NamedMonitorInfo;
	typedef struct AP_NamedJavaApplicationInfo_API AP_NamedJavaApplicationInfo;
	typedef struct AP_NamedEventTypeInfo_API AP_NamedEventTypeInfo;
	typedef struct AP_NamedTimerInfo_API AP_NamedTimerInfo;
	typedef struct AP_NamedContextInfo_API AP_NamedContextInfo;
	typedef struct AP_NamedAggregateInfo_API AP_NamedAggregateInfo;
	typedef struct AP_NamedReceiverInfo_API AP_NamedReceiverInfo;
	typedef struct AP_NamedPluginReceiverInfo_API AP_NamedPluginReceiverInfo;
	typedef struct AP_EngineInfo_API AP_EngineInfo;
	typedef struct AP_EngineManagement_API AP_EngineManagement;
#endif /* __cplusplus */


	/**
	 * Event functions API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_Event_Functions
	{
		/**
		* Retrieve the event's type and its contents as a string.
		*
		* @return C style string with the event's textual representation.
		* Note that these strings should be encoded in UTF-8.
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* (AP_ENGINE_CLIENT_CALL* getText)(AP_Event* event);
		/**
		* Retrieve the event's channel.  This currently only has meaning for
		* events which have been sourced from the correlator, in which case
		* it is set to the name of the channel the event was emitted from, or
		* "" (the empty string) for the wildcard channel.
		*
		* @return The channel of the event, encoded in UTF-8.
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* (AP_ENGINE_CLIENT_CALL* getChannel)(AP_Event* event);
		/**
		* Retrieve the event's timestamp.  This timestamp currently only has
		* meaning for events which have been sourced from the correlator, in
		* which case it is set to the correlator time at which it was created,
		* or the value it was set to explicitly by code running within the
		* correlator.
		*
		* @return the timestamp of the event, in floating point seconds since
		* the Unix epoch.
		* @deprecated The C++ API should be used in preference
		*/
		double (AP_ENGINE_CLIENT_CALL* getTime)(AP_Event* event);
	};

	/**
	 * Event
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_Event_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_Event_Functions* functions;
	};

	struct AP_Event_Stubs
	{
		struct AP_Event_API api;

#ifdef __cplusplus
		const AP_char8* getText() {
			return api.functions->getText(this);
		}

		const AP_char8* getChannel() {
			return api.functions->getChannel(this);
		}

		double getTime() {
			return api.functions->getTime(this);
		}
#endif /* __cplusplus */
	};


	/**
	 * EventSupplier function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_EventSupplier_Functions
	{
		/**
		* Disconnect the EventSupplier from its consumer.
		*
		* Disconnection is automatically performed as part of 
		* AP_DeleteEventSupplier so in most cases there is no need to call 
		* this method explicitly. 
		*
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* disconnect)(AP_EventSupplier* supplier);
	};

	/**
	 * Event supplier
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_EventSupplier_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_EventSupplier_Functions* functions;
	};

	struct AP_EventSupplier_Stubs {
		struct AP_EventSupplier_API api;

#ifdef __cplusplus
		void disconnect() {
			api.functions->disconnect(this);
		}
#endif /*__cplusplus */
	};
	
	/**
	 * EventConsumer function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_EventConsumer_Functions
	{
		/**
		* This function must be defined by the client to
		* enable receiving of events. This function is called by an
		* EventSupplier.
		*
		* Note that EventSuppliers are not reentrant so
		* calling the disconnect method on the calling
		* EventSupplier is not permitted from within the
		* sendEvents implementation.
		*
		* @param events An array of pointers to Event objects.
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* sendEvents)(AP_EventConsumer* consumer, AP_Event** events);
	};

	/**
	 * EventConsumer
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_EventConsumer_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_EventConsumer_Functions* functions;
	};

	struct AP_EventConsumer_Stubs
	{
		struct AP_EventConsumer_API api;

#ifdef __cplusplus
		void sendEvents(AP_Event** events) {
			api.functions->sendEvents(this, events);
		}
#endif /* __cplusplus */
	};

	/**
	 * DisconnectableEventConsumer function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_DisconnectableEventConsumer_Functions
	{
		/**
		* Used to inform the consumer that it is not going 
		* to be sent any more events.
		*
		* @param A string giving the reason why the consumer 
		* is being disconnected. May be NULL.
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* disconnect)(AP_DisconnectableEventConsumer* consumer, const AP_char8* reason);
	};

	/**
	 * DisconnectableEventConsumer
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_DisconnectableEventConsumer_API
	{
		void *reserved;
		/** Pointer to function API */
		struct AP_DisconnectableEventConsumer_Functions* functions;
		/** Pointer to EventConsumer facade for this object */
		AP_EventConsumer* s_EventConsumer;
	};

	struct AP_DisconnectableEventConsumer_Stubs
	{
	  struct AP_DisconnectableEventConsumer_API api;

#ifdef __cplusplus
	  void disconnect(const AP_char8* reason) {
	    api.functions->disconnect(this, reason);
	  }

	  void sendEvents(AP_Event** events) {
	    api.s_EventConsumer->api.functions->sendEvents(api.s_EventConsumer, events);
	  }
#endif /* __cplusplus */
	};

	/**
	 * MonitorScript function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_MonitorScript_Functions
	{
		/**
		* Retrieve the text of a MonitorScript fragment as a string.
		*
		* @return C style string containing the text of the
		* MonitorScript fragment. Note that this string should
		* be encoded in UTF-8.
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* (AP_ENGINE_CLIENT_CALL* getText)(AP_MonitorScript* script);
	};

	/**
	 * Monitorscript
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_MonitorScript_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_MonitorScript_Functions* functions;
	};

	struct AP_MonitorScript_Stubs
	{
		struct AP_MonitorScript_API api;

#ifdef __cplusplus
		const AP_char8* getText() {
			return api.functions->getText(this);
		};
#endif /*__cplusplus */
	};


	/**
	 * EngineStatus function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_EngineStatus_Functions
	{
		/**
		 * Format the status into the given buffer.
		 * @deprecated The C++ API should be used in preference
		 */
		AP_bool (AP_ENGINE_CLIENT_CALL* print)(AP_EngineStatus* status, AP_char8* buf, AP_uint32 size);
	
		/**
		* Get the time in ms that the Engine has been running
		* for.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint64 (AP_ENGINE_CLIENT_CALL* getUptime)(AP_EngineStatus* status);

		/**
		* Get the number of monitors defined in the Engine.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumMonitors)(AP_EngineStatus* status);

		/**
		* Get the number of monitor processes or active
		* sub-monitors. If a monitor spawns it creates a new
		* process.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumProcesses)(AP_EngineStatus* status);

		/**
		* Get the number of active listeners.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumListeners)(AP_EngineStatus* status);

		/**
		* Get the number of event types defined.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumEventTypes)(AP_EngineStatus* status);

		/**
		 * Get the total number of events waiting on the route queues (value
		 * of 'rq' in the correlator status line).
		 * @deprecated The C++ API should be used in preference
		 */
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumQueuedFastTrack)(AP_EngineStatus* status);

		/**
		 * Get the total number of events waiting on the input queues (value
		 * of 'iq' in the correlator status line).
		 * @deprecated The C++ API should be used in preference
		 */
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumQueuedInput)(AP_EngineStatus* status);

		/**
		* Get the number of events received since the Engine
		* started (including those discarded because they
		* were invalid)..
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint64 (AP_ENGINE_CLIENT_CALL* getNumReceived)(AP_EngineStatus* status);

		/**
		* Get the number of events received on the internal input
		* queue since the Engine started.
		*/
		AP_uint64 (AP_ENGINE_CLIENT_CALL* getNumFastTracked)(AP_EngineStatus* status);

		/**
		* Get the number of event consumers connected to the
		* engine.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumConsumers)(AP_EngineStatus* status);

		/**
		* Get the number of events waiting on the output queue.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumOutputEventsQueued)(AP_EngineStatus* status);

		/**
		* Gets the number of output events which have been
		* put onto the output queue. This corresponds to the
		* number of MonitorScript emit commands executed.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint64 (AP_ENGINE_CLIENT_CALL* getNumOutputEventsCreated)(AP_EngineStatus* status);

		/**
		* This is the number of output events sent out by the
		* correlator process. This differs from getNumOutEventsCreated
		* since events can be of interest to a varying number of 
		* consumers.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint64 (AP_ENGINE_CLIENT_CALL* getNumOutputEventsSent)(AP_EngineStatus* status);

		/**
		* Get the number of java applications defined in the Engine.
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumJavaApplications)(AP_EngineStatus* status);

		/**
		* Get the number of events taken off the input queue
		* and processed since the Engine started.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint64 (AP_ENGINE_CLIENT_CALL* getNumProcessed)(AP_EngineStatus* status);	

		/**
		* Get the number of active listeners.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumSubListeners)(AP_EngineStatus* status);	

		/**
		* Get the number of active contexts.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumContexts)(AP_EngineStatus* status);	

		/**
		 * Get the name of the most backed up (slowest) context
		 * @deprecated The C++ API should be used in preference
		 */
		const AP_char8* (AP_ENGINE_CLIENT_CALL* getMostBackedUpInput)(AP_EngineStatus* status);

		/**
		 * Get the queue size of the most backed up (slowest) context
		 * @deprecated The C++ API should be used in preference
		 */
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getMostBackedUpQueueSize)(AP_EngineStatus* status);	

		/**
		 * Get the name of the slowest receiver
		 * @deprecated The C++ API should be used in preference
		 */
		const AP_char8* (AP_ENGINE_CLIENT_CALL* getSlowestReceiver)(AP_EngineStatus* status);

		/**
		 * Get the queue size of the slowest receiver
		 * @deprecated The C++ API should be used in preference
		 */
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getSlowestReceiverQueueSize)(AP_EngineStatus* status);	

		/**
		* Sets arguments with the next status item name and value 
		*
		* Sample usage:
		*	void *opaque_state = 0;
		*	const char *name;
		*	const char *value;
		*	while (status->getNextStatusItem(&name, &value, &opaque_state)) {
		*		//Do stuff with name,value
		*	}
		*	status->disposeIterator(opaque_state);

		* @return 1 if the name and the value were set, or 0
		* if there were no more status items, in which case the 
		* name and the value arguments are not set
		* @param name contains the name of the status item upon 
		* return of the function
		* @param value contains the value of the status item upon
		* return of the function
		* @param opaque_state a state pointer which must be initiazed to null 
		* when the function is called the first time. This pointer must be
		* free'd by calling disposeIterator.
		 * @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNextStatusItem)(AP_EngineStatus* status, const char **name, const char **value, void **opaque_state);

		/**
		 * Get the status value with the specified key.
		 * @deprecated The C++ API should be used in preference
		 */
		const AP_char8* (AP_ENGINE_CLIENT_CALL* getStatusValue)(AP_EngineStatus* status, const char *key);

		/**
		 * Get the status value with the specified key, as an integer.
		 * @deprecated The C++ API should be used in preference
		 */
		AP_uint64 (AP_ENGINE_CLIENT_CALL* getStatusIntValue)(AP_EngineStatus* status, const char *key);

		/**
		 * Delete the supplied AP_EngineStatus (returned from a previous call to AP_EngineManagement->getStatus())
		 * @deprecated The C++ API should be used in preference
		 */
		void (AP_ENGINE_CLIENT_CALL* deleteStatus)(AP_EngineStatus *status);

		/**
		 * Copy the supplied AP_EngineStatus (returned from a previous call to AP_EngineManagement->getStatus())
		 * @deprecated The C++ API should be used in preference
		 */
		AP_EngineStatus *(AP_ENGINE_CLIENT_CALL* copyStatus)(AP_EngineStatus *status);

		/**
		 * Copy an opaque_state as returned by getNextStatusItem
		 * @deprecated The C++ API should be used in preference
		 */
		void *(AP_ENGINE_CLIENT_CALL* copyIterator)(AP_EngineStatus* status, void *opaque_state);

		/**
		 * Delete the opaque_state as returned by getNextStatusItem
		 * @deprecated The C++ API should be used in preference
		 */
		void (AP_ENGINE_CLIENT_CALL* disposeIterator)(AP_EngineStatus* status, void *opaque_state);

		/**
		* Get a user value of type AP_int64
		*
		* @exception EngineException
		* @return The user status.
		* @deprecated The C++ API should be used in preference
		*/
		AP_int64 (AP_ENGINE_CLIENT_CALL* getUserInt)(AP_EngineStatus* status, const char* key);

		/**
		* Get a user value of type AP_int64 or the alt value if it cannot be found
		*
		* @exception EngineException
		* @return The user status.
		* @deprecated The C++ API should be used in preference
		*/
		AP_int64 (AP_ENGINE_CLIENT_CALL* getUserIntOr)(AP_EngineStatus* status, const char* key, AP_int64 alt);

		/**
		* Get a user value of type AP_float64
		*
		* @exception EngineException
		* @return The user status.
		* @deprecated The C++ API should be used in preference
		*/
		AP_float64(AP_ENGINE_CLIENT_CALL* getUserFloat)(AP_EngineStatus* status, const char* key);
		/**
		* Get a user value of type AP_float64 or the alt value if it cannot be found
		*
		* @exception EngineException
		* @return The user status.
		* @deprecated The C++ API should be used in preference
		*/
		AP_float64(AP_ENGINE_CLIENT_CALL* getUserFloatOr)(AP_EngineStatus* status, const char* key, AP_float64 alt);

		/**
		* Get a user value of type AP_bool
		*
		* @exception EngineException
		* @return The user status.
		* @deprecated The C++ API should be used in preference
		*/
		AP_bool(AP_ENGINE_CLIENT_CALL* getUserBool)(AP_EngineStatus* status, const char* key);

		/**
		* Get a user value of type AP_bool or the alt value if it cannot be found
		*
		* @exception EngineException
		* @return The user status.
		* @deprecated The C++ API should be used in preference
		*/
		AP_bool(AP_ENGINE_CLIENT_CALL* getUserBoolOr)(AP_EngineStatus* status, const char* key, AP_bool alt);

		/**
		* Get a user value of type AP_char8*
		*
		* @exception EngineException
		* @return The user status.
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8*(AP_ENGINE_CLIENT_CALL* getUserString)(AP_EngineStatus* status, const char* key);

		/**
		* Get a user value of type AP_char8* or the alt value if it cannot be found
		*
		* @exception EngineException
		* @return The user status.
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8*(AP_ENGINE_CLIENT_CALL* getUserStringOr)(AP_EngineStatus* status, const char* key, const AP_char8* alt);
	};

	/**
	 * EngineStatus
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_EngineStatus_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_EngineStatus_Functions* functions;
	};

	struct AP_EngineStatus_Stubs
	{
		struct AP_EngineStatus_API api;

#ifdef __cplusplus
		AP_bool print(AP_char8* buf, AP_uint32 size) {
			return api.functions->print(this,buf,size);
		}

		AP_uint64 getUptime() {
			return api.functions->getUptime(this);
		}

		AP_uint32 getNumMonitors() {
			return api.functions->getNumMonitors(this);
		}

		AP_uint32 getNumProcesses() {
			return api.functions->getNumProcesses(this);
		}

		AP_uint32 getNumJavaApplications() {
			return api.functions->getNumJavaApplications(this);
		}

		AP_uint32 getNumListeners() {
			return api.functions->getNumListeners(this);
		}

		AP_uint32 getNumEventTypes() {
			return api.functions->getNumEventTypes(this);
		}

		AP_uint32 getNumQueuedFastTrack() {
			return api.functions->getNumQueuedFastTrack(this);
		}

		AP_uint32 getNumQueuedInput() {
			return api.functions->getNumQueuedInput(this);
		}

		AP_uint64 getNumReceived() {
			return api.functions->getNumReceived(this);
		}

		AP_uint64 getNumFastTracked() {
			return api.functions->getNumFastTracked(this);
		}

		AP_uint32 getNumConsumers() {
			return api.functions->getNumConsumers(this);
		}

		AP_uint32 getNumOutputEventsQueued() {
			return api.functions->getNumOutputEventsQueued(this);
		}

		AP_uint64 getNumOutputEventsCreated() {
			return api.functions->getNumOutputEventsCreated(this);
		}

		AP_uint64 getNumOutputEventsSent() {
			return api.functions->getNumOutputEventsSent(this);
		}

		AP_uint64 getNumProcessed() {
			return api.functions->getNumProcessed(this);
		}

		AP_uint32 getNumSubListeners() {
			return api.functions->getNumSubListeners(this);
		}

		AP_uint32 getNumContexts() {
			return api.functions->getNumContexts(this);
		}

		AP_uint32 getNextStatusItem(const char **name, const char **value, void **opaque_state) {
			return api.functions->getNextStatusItem(this, name, value, opaque_state);
		}

		AP_int64 getUserInt(const char* key) {
			return api.functions->getUserInt(this, key);
		}
		AP_int64 getUserIntOr(const char* key, AP_int64 alt) {
			return api.functions->getUserIntOr(this, key, alt);
		}

		AP_float64 getUserFloat(const char* key) {
			return api.functions->getUserFloat(this, key);
		}
		AP_float64 getUserFloatOr(const char* key, AP_float64 alt) {
			return api.functions->getUserFloatOr(this, key, alt);
		}

		AP_bool getUserBool(AP_EngineStatus* status, const char* key) {
			return api.functions->getUserBool(this, key);
		}
		AP_bool getUserBoolOr(AP_EngineStatus* status, const char* key, AP_bool alt) {
			return api.functions->getUserBoolOr(this, key, alt);
		}

		const AP_char8* getUserString(AP_EngineStatus* status, const char* key) {
			return api.functions->getUserString(this, key);
		}
		const AP_char8* getUserStringOr(AP_EngineStatus* status, const char* key, const AP_char8* alt) {
			return api.functions->getUserStringOr(this, key, alt);
		}

		void disposeIterator(void *opaque_state) {
			api.functions->disposeIterator(this, opaque_state);
		}
		const AP_char8* getMostBackedUpInput() {
			return api.functions->getMostBackedUpInput(this);
		}
		AP_uint32 getMostBackedUpQueueSize() {
			return api.functions->getMostBackedUpQueueSize(this);
		}
		
		const AP_char8* getSlowestReceiver() {
			return api.functions->getSlowestReceiver(this);
		}

		AP_uint32 getSlowestReceiverQueueSize() {
			return api.functions->getSlowestReceiverQueueSize(this);
		}
		

#endif /*__cplusplus */
	};

	/**
	 * NameInfo function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NameInfo_Functions
	{
		/**
		* Name, excluding package, for example "MyEvent".
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* (AP_ENGINE_CLIENT_CALL* getName)(AP_NameInfo* info);

		/**
		* Package name, for example "com.apamax", or an 
		* empty string if in the  default package.
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* (AP_ENGINE_CLIENT_CALL* getPackage)(AP_NameInfo* info);

		/**
		* Fully qualified name, for example "com.apamax.MyEvent"
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* (AP_ENGINE_CLIENT_CALL* getFullyQualifiedName)(AP_NameInfo* info);
	};

	/**
	 * NameInfo
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NameInfo_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_NameInfo_Functions* functions;
	};

	struct AP_NameInfo_Stubs
	{
		struct AP_NameInfo_API api;

#ifdef __cplusplus
		const AP_char8* getName() {
			return api.functions->getName(this);
		}

		const AP_char8* getPackage() {
			return api.functions->getPackage(this);
		}

		const AP_char8* getFullyQualifiedName() {
			return api.functions->getFullyQualifiedName(this);
		}
#endif /*__cplusplus */
	};


	/**
	 * NamedMonitorInfo function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedMonitorInfo_Functions
	{
		/**
		* Gets the number of sub-monitors belonging to this monitor.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumSubMonitors)(AP_NamedMonitorInfo* info);
	};

	/**
	 * NamedMonitorInfo facade for NamedInfo object
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedMonitorInfo_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_NamedMonitorInfo_Functions* functions;
		/** Pointer to NameInfo facade for this object */
		AP_NameInfo* s_NameInfo;
	};

	struct AP_NamedMonitorInfo_Stubs
	{
		struct AP_NamedMonitorInfo_API api;

#ifdef __cplusplus
		AP_uint32 getNumSubMonitors() {
			return api.functions->getNumSubMonitors(this);
		}

		const AP_char8* getName() {
			return api.s_NameInfo->api.functions->getName(api.s_NameInfo);
		}

		const AP_char8* getPackage() {
			return api.s_NameInfo->api.functions->getPackage(api.s_NameInfo);
		}

		const AP_char8* getFullyQualifiedName() {
			return api.s_NameInfo->api.functions->getFullyQualifiedName(api.s_NameInfo);
		}
#endif /*__cplusplus */
	};


	/**
	 * NamedJavaApplicationInfo function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedJavaApplicationInfo_Functions
	{
		/**
		* Gets the number of listeners belonging to this application.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumListeners)(AP_NamedJavaApplicationInfo* info);
	};

	/**
	 * NamedJavaApplicationInfo facade for NamedInfo object
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedJavaApplicationInfo_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_NamedJavaApplicationInfo_Functions* functions;
		/** Pointer to NameInfo facade for this object */
		AP_NameInfo* s_NameInfo;
	};

	struct AP_NamedJavaApplicationInfo_Stubs
	{
		struct AP_NamedJavaApplicationInfo_API api;

#ifdef __cplusplus
		AP_uint32 getNumListeners() {
			return api.functions->getNumListeners(this);
		}

		const AP_char8* getName() {
			return api.s_NameInfo->api.functions->getName(api.s_NameInfo);
		}

		const AP_char8* getPackage() {
			return api.s_NameInfo->api.functions->getPackage(api.s_NameInfo);
		}

		const AP_char8* getFullyQualifiedName() {
			return api.s_NameInfo->api.functions->getFullyQualifiedName(api.s_NameInfo);
		}
#endif /*__cplusplus */
	};


	/**
	 * NamedEventTypeInfo function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedEventTypeInfo_Functions
	{
		/**
		* Gets the number of event templates for this event type.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumEventTemplates)(AP_NamedEventTypeInfo* info);
	};

	/**
	 * NamedEventTypeInfo facade for NamedInfo object
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedEventTypeInfo_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_NamedEventTypeInfo_Functions* functions;
		/** Pointer to NameInfo facade for this object */
		AP_NameInfo* s_NameInfo;
	};

	struct AP_NamedEventTypeInfo_Stubs
	{
		struct AP_NamedEventTypeInfo_API api;

#ifdef __cplusplus
		AP_uint32 getNumEventTemplates() {
			return api.functions->getNumEventTemplates(this);
		}

		const AP_char8* getName() {
			return api.s_NameInfo->api.functions->getName(api.s_NameInfo);
		}

		const AP_char8* getPackage() {
			return api.s_NameInfo->api.functions->getPackage(api.s_NameInfo);
		}

		const AP_char8* getFullyQualifiedName() {
			return api.s_NameInfo->api.functions->getFullyQualifiedName(api.s_NameInfo);
		}
#endif /*__cplusplus */
	};


	/**
	 * NamedTimerInfoInfo function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedTimerInfo_Functions
	{
		/**
		* Gets the number of timers for this timer type.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumTimers)(AP_NamedTimerInfo* info);
	};

	/**
	 * NamedTimerInfoInfo facade for NamedInfo object
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedTimerInfo_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_NamedTimerInfo_Functions* functions;
		/** Pointer to NameInfo facade for this object */
		AP_NameInfo* s_NameInfo;
	};

	struct AP_NamedTimerInfo_Stubs
	{
		struct AP_NamedTimerInfo_API api;

#ifdef __cplusplus
		AP_uint32 getNumTimers() {
			return api.functions->getNumTimers(this);
		}

		const AP_char8* getName() {
			return api.s_NameInfo->api.functions->getName(api.s_NameInfo);
		}

		const AP_char8* getPackage() {
			return api.s_NameInfo->api.functions->getPackage(api.s_NameInfo);
		}

		const AP_char8* getFullyQualifiedName() {
			return api.s_NameInfo->api.functions->getFullyQualifiedName(api.s_NameInfo);
		}
#endif /*__cplusplus */
	};


	/**
	 * NamedContextInfo function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedContextInfo_Functions {
		/**
		* Gets the number of sub-monitors belonging to this context.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumSubMonitors)(AP_NamedContextInfo* info);

		/**
		* Gets the queue size of this context.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getQueueSize)(AP_NamedContextInfo* info);

		/**
		 * Gets the channels to which this context is subscribed.
		* @deprecated The C++ API should be used in preference
		 */
		char const *const *(AP_ENGINE_CLIENT_CALL *getChannels)(AP_NamedContextInfo *info);
	};

	/**
	 * NamedContextInfo facade for NamedInfo object
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedContextInfo_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_NamedContextInfo_Functions* functions;
		/** Pointer to NameInfo facade for this object */
		AP_NameInfo* s_NameInfo;
	};

	struct AP_NamedContextInfo_Stubs
	{
		struct AP_NamedContextInfo_API api;

#ifdef __cplusplus
		const AP_char8* getName() {
			return api.s_NameInfo->api.functions->getName(api.s_NameInfo);
		}

		AP_uint32 getNumSubMonitors() {
			return api.functions->getNumSubMonitors(this);
		}

		AP_uint32 getQueueSize() {
			return api.functions->getQueueSize(this);
		}

		char const *const *getChannels() {
			return api.functions->getChannels(this);
		}

#endif /*__cplusplus */
	};

	/**
	 * NamedAggregateInfo function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedAggregateInfo_Functions
	{
		void** placeholder;
	};

	/**
	 * NamedAggregateInfo facade for NamedInfo object
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedAggregateInfo_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_NamedAggregateInfo_Functions* functions;
		/** Pointer to NameInfo facade for this object */
		AP_NameInfo* s_NameInfo;
	};

	struct AP_NamedAggregateInfo_Stubs
	{
		struct AP_NamedAggregateInfo_API api;

#ifdef __cplusplus
		const AP_char8* getName() {
			return api.s_NameInfo->api.functions->getName(api.s_NameInfo);
		}

		const AP_char8* getPackage() {
			return api.s_NameInfo->api.functions->getPackage(api.s_NameInfo);
		}

		const AP_char8* getFullyQualifiedName() {
			return api.s_NameInfo->api.functions->getFullyQualifiedName(api.s_NameInfo);
		}
#endif /*__cplusplus */
	};

	/**
	 * NamedReceiverInfo function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedReceiverInfo_Functions
	{
		/**
		* Gets the queue size of this receiver.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getQueueSize)(AP_NamedReceiverInfo* info);

		/**
		 * Gets the channels to which this context is subscribed.
		 * @deprecated The C++ API should be used in preference
		 */
		char const *const *(AP_ENGINE_CLIENT_CALL *getChannels)(AP_NamedReceiverInfo *info);

		/**
		 * Get the address of this receiver
		 * @deprecated The C++ API should be used in preference
		 */
		const AP_char8 *(AP_ENGINE_CLIENT_CALL *getAddress)(AP_NamedReceiverInfo *info);
	};

	/**
	 * NamedReceiverInfo facade for NamedInfo object
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedReceiverInfo_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_NamedReceiverInfo_Functions* functions;
		/** Pointer to NameInfo facade for this object */
		AP_NameInfo* s_NameInfo;
	};

	struct AP_NamedReceiverInfo_Stubs
	{
		struct AP_NamedReceiverInfo_API api;

#ifdef __cplusplus
		const AP_char8 *getName() {
			return api.s_NameInfo->api.functions->getName(api.s_NameInfo);
		}

		const AP_char8 *getAddress() {
			return api.functions->getAddress(this);
		}

		AP_uint32 getQueueSize() {
			return api.functions->getQueueSize(this);
		}

		char const *const *getChannels() {
			return api.functions->getChannels(this);
		}

#endif /*__cplusplus */
	};

	/**
	 * NamedPluginReceiverInfo function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedPluginReceiverInfo_Functions
	{
		/**
		* Gets the queue size of this receiver.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getQueueSize)(AP_NamedPluginReceiverInfo* info);

		/**
		 * Gets the channels to which this context is subscribed.
		 * @deprecated The C++ API should be used in preference
		 */
		char const *const *(AP_ENGINE_CLIENT_CALL *getChannels)(AP_NamedPluginReceiverInfo *info);
	};

	/**
	 * NamedPluginReceiverInfo facade for NamedInfo object
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_NamedPluginReceiverInfo_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_NamedPluginReceiverInfo_Functions* functions;
		/** Pointer to NameInfo facade for this object */
		AP_NameInfo* s_NameInfo;
	};

	struct AP_NamedPluginReceiverInfo_Stubs
	{
		struct AP_NamedPluginReceiverInfo_API api;

#ifdef __cplusplus
		const AP_char8 *getName() {
			return api.s_NameInfo->api.functions->getName(api.s_NameInfo);
		}

		AP_uint32 getQueueSize() {
			return api.functions->getQueueSize(this);
		}

		char const *const *getChannels() {
			return api.functions->getChannels(this);
		}

#endif /*__cplusplus */
	};

	/**
	 * EngineInfo function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_EngineInfo_Functions
	{
		/**
		* Gets the number of monitors in the engine.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumMonitors)(AP_EngineInfo* info);

		/**
		* Gets the number of event types in the engine.
		* @deprecated The C++ API should be used in preference
		*/					
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumEventTypes)(AP_EngineInfo* info);

		/**
		* Gets the number of timers in the engine.
		* @deprecated The C++ API should be used in preference
		*/					
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumTimers)(AP_EngineInfo* info);

		/**
		* Returns information about the monitors in the engine,
		* in the form of a NULL terminated array of pointers to
		* AP_NamedMonitorInfo instances. The size of the array can be found
		* by calling getNumMonitors (or looking for the
		* NULL terminator).
		* @deprecated The C++ API should be used in preference
		*/					
		AP_NamedMonitorInfo** (AP_ENGINE_CLIENT_CALL* getMonitors)(AP_EngineInfo* info);

		/**
		* Returns information about the event types in the engine,
		* in the form of a NULL terminated array of pointers to
		* AP_NamedEventTypeInfo instances. The size of the array can be 
		* found by calling getNumEventTypes (or looking for the
		* NULL terminator).
		* @deprecated The C++ API should be used in preference
		*/					
		AP_NamedEventTypeInfo** (AP_ENGINE_CLIENT_CALL* getEventTypes)(AP_EngineInfo* info);

		/**
		* Returns information about the timers in the engine,
		* in the form of a NULL terminated array of pointers to
		* AP_NamedTimerInfo instances. The size of the array can be 
		* found by calling getNumTimers (or looking for the
		* NULL terminator).
		* @deprecated The C++ API should be used in preference
		*/					
		AP_NamedTimerInfo** (AP_ENGINE_CLIENT_CALL* getTimers)(AP_EngineInfo* info);

		/**
		* Gets the number of Java applications in the engine.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumJavaApplications)(AP_EngineInfo* info);

		/**
		* Returns information about the Java applications in the
		* engine, in the form of a NULL terminated array of pointers
		* to AP_NamedJavaApplicationInfo instances. The size of the array can
		* be found by calling getNumJavaApplications (or looking for
		* the NULL terminator).
		* @deprecated The C++ API should be used in preference
		*/					
		AP_NamedJavaApplicationInfo** (AP_ENGINE_CLIENT_CALL* getJavaApplications)(AP_EngineInfo* info);

		/**
		* Gets the number of contexts in the engine.
		* @deprecated The C++ API should be used in preference
		*/
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumContexts)(AP_EngineInfo* info);

		/**
		* Returns information about the contexts in the
		* engine, in the form of a NULL terminated array of pointers
		* to AP_NamedContextInfo instances. The size of the array can
		* be found by calling getNumContexts (or looking for
		* the NULL terminator).
		* @deprecated The C++ API should be used in preference
		*/					
		AP_NamedContextInfo** (AP_ENGINE_CLIENT_CALL* getContexts)(AP_EngineInfo* info);

		/**
		* Gets the number of aggregate functions in the engine.
		* @deprecated The C++ API should be used in preference
		*/					
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumAggregates)(AP_EngineInfo* info);

		/**
		* Returns information about the aggregate functions in the
		* engine, in the form of a NULL terminated array of pointers
		* to AP_NamedAggregateInfo instances. The size of the array can be
		* found by calling getNumAggregates (or looking for the
		* NULL terminator).
		* @deprecated The C++ API should be used in preference
		*/					
		AP_NamedAggregateInfo** (AP_ENGINE_CLIENT_CALL* getAggregates)(AP_EngineInfo* info);

		/**
		* Gets the number of receivers connected to the engine
		* @deprecated The C++ API should be used in preference
		*/					
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumReceivers)(AP_EngineInfo* info);

		/**
		* Returns information about the receivers connected to the
		* engine, in the form of a NULL terminated array of pointers
		* to AP_NamedReceiverInfo instances. The size of the array can be
		* found by calling getNumReceivers (or looking for the
		* NULL terminator).
		* @deprecated The C++ API should be used in preference
		*/					
		AP_NamedReceiverInfo** (AP_ENGINE_CLIENT_CALL* getReceivers)(AP_EngineInfo* info);

		/**
		* Gets the number of plugin receivers in the engine
		* @deprecated The C++ API should be used in preference
		*/					
		AP_uint32 (AP_ENGINE_CLIENT_CALL* getNumPluginReceivers)(AP_EngineInfo* info);

		/**
		* Returns information about the plugin receivers in the
		* engine, in the form of a NULL terminated array of pointers
		* to AP_NamedReceiverInfo instances. The size of the array can be
		* found by calling getNumPluginReceivers (or looking for the
		* NULL terminator).
		* @deprecated The C++ API should be used in preference
		*/					
		AP_NamedPluginReceiverInfo** (AP_ENGINE_CLIENT_CALL* getPluginReceivers)(AP_EngineInfo* info);
	};

	/**
	 * EngineInfo
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_EngineInfo_API
	{
		void* reserved;
		/** Pointer to function API */
		struct AP_EngineInfo_Functions* functions;
	};

	struct AP_EngineInfo_Stubs
	{
		struct AP_EngineInfo_API api;

#ifdef __cplusplus
		AP_uint32 getNumMonitors() {
			return api.functions->getNumMonitors(this);
		}

		AP_uint32 getNumEventTypes() {
			return api.functions->getNumEventTypes(this);
		}

		AP_uint32 getNumTimers() {
			return api.functions->getNumTimers(this);
		}

		AP_NamedMonitorInfo** getMonitors() {
			return api.functions->getMonitors(this);
		}

		AP_NamedEventTypeInfo** getEventTypes() {
			return api.functions->getEventTypes(this);
		}

		AP_NamedTimerInfo** getTimers() {
			return api.functions->getTimers(this);
		}

		AP_uint32 getNumJavaApplications() {
			return api.functions->getNumJavaApplications(this);
		}

		AP_NamedJavaApplicationInfo** getJavaApplications() {
			return api.functions->getJavaApplications(this);
		}

		AP_uint32 getNumContexts() {
			return api.functions->getNumContexts(this);
		}

		AP_NamedContextInfo** getContexts() {
			return api.functions->getContexts(this);
		}

		AP_uint32 getNumAggregates() {
			return api.functions->getNumAggregates(this);
		}
		
		AP_NamedAggregateInfo** getAggregates() {
			return api.functions->getAggregates(this);
		}

		AP_uint32 getNumReceivers() {
			return api.functions->getNumReceivers(this);
		}
		
		AP_NamedReceiverInfo** getReceivers() {
			return api.functions->getReceivers(this);
		}

		AP_uint32 getNumPluginReceivers() {
			return api.functions->getNumPluginReceivers(this);
		}
		
		AP_NamedPluginReceiverInfo** getPluginReceivers() {
			return api.functions->getPluginReceivers(this);
		}

#endif /*__cplusplus */
	};


	/** 
	 * The mode that attachAsEventConsumerTo and detachAsEventConsumerFrom operate in 
	 * @deprecated The C++ API should be used in preference
	 */
	enum AP_EngineManagement_ConnectMode {
		/** For backwards compatibility, single serial connection */
		CONNECT_LEGACY,
		/** Allow parallel connections on multiple channels */
		CONNECT_PARALLEL
	};

	typedef enum AP_EngineManagement_ConnectMode AP_EngineManagement_ConnectMode;


	/**
	 * EngineManagement function API
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_EngineManagement_Functions
	{
		/**
		* Inject MonitorScript text into the Engine.
		*
		* @param script MonitorScript text to be injected.
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* injectMonitorScript)(AP_EngineManagement* engine, AP_MonitorScript* script);

		/**
		* Injects a Java application (a jar) into the engine.
		*
		* @param jarbytes A pointer to the array of bytes
		* containing the jar
		* @param size The size of the jar
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* injectJava)(AP_EngineManagement* engine, const AP_uint8* jarbytes, AP_uint32 size);

		/**
		* Injects a CDP (Correlator Deployment Package file) into the engine.
		*
		* @param cdpbytes A pointer to the array of bytes
		* containing the CDP file
		* @param size The size of the CDP file
		* @param filename The name of the CDP file
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* injectCDP)(AP_EngineManagement* engine, const AP_uint8* cdpbytes, AP_uint32 size, const char*filename);

		/**
		* Delete a named object from the Engine.
		*
		* @param name	The name of the object to be deleted.
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* deleteName)(AP_EngineManagement* engine, const AP_char8* name);

		/**
		* Force deletion of a named object from the Engine.
		*
		* @param name	The name of the object to be deleted.
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* forceDeleteName)(AP_EngineManagement* engine, const AP_char8* name);

		/**
		* Kill a named monitor in the Engine.
		*
		* @param name	The name of the monitor to be killed.
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* killName)(AP_EngineManagement* engine, const AP_char8* name);

		/**
		* Deletes everything from the engine.
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* deleteAll)(AP_EngineManagement* engine);

		/**
		* Get the Engine's current operational status. Use 
		* AP_DeleteEngineStatus to delete the returned object when
		* it is no longer needed.
		*
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		AP_EngineStatus* (AP_ENGINE_CLIENT_CALL* getStatus)(AP_EngineManagement* engine);

		/**
		* Connect an event receiver to the Engine.
		*
		* @param consumer The AP_EventConsumer to connect to the
		* Engine.
		* @param channels An array of names representing the
		* channels to subscribe to. This is a null-terminated array
		* of pointers to zero-terminated char arrays. If it is null or
		* empty subscribe to all channels. Note that these channel
		* names should be encoded in UTF-8.
		* @return A pointer to an AP_EventSupplier resource, which the caller 
		* is responsible for freeing, using AP_DeleteEventSupplier(). 
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		AP_EventSupplier* (AP_ENGINE_CLIENT_CALL* connectEventConsumer)(AP_EngineManagement* engine, AP_EventConsumer* consumer, const AP_char8* const* channels);

		/**
		* Connect a disconnectable event receiver to the Engine.
		*
		* @param consumer The AP_EventConsumer to connect to the
		* Engine.
		* @param channels An array of names representing the
		* channels to subscribe to. This is a null-terminated array
		* of pointers to zero-terminated char arrays. If it is null or
		* empty subscribe to all channels. Note that these channel
		* names should be encoded in UTF-8.
		* @return A pointer to an AP_EventSupplier resource, which the caller 
		* is responsible for freeing, using AP_DeleteEventSupplier(). 
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		AP_EventSupplier* (AP_ENGINE_CLIENT_CALL* connectDisconnectableEventConsumer)(AP_EngineManagement* engine, AP_DisconnectableEventConsumer* consumer, const AP_char8* const* channels);

		/**
		* Connect a disconnectable event receiver to the Engine. 
		* It informs the Engine it can disconnect
		* this receiver if it is slow. This may be overridden by 
		* correlator configuration (to always disconnect us), or
		* by a previous consumer to the same correlator (the setting
		* is a per connected component setting).
		*
		* @param consumer The AP_EventConsumer to connect to the
		* Engine.
		* @param channels An array of names representing the
		* channels to subscribe to. This is a null-terminated array
		* of pointers to zero-terminated char arrays. If it is null or
		* empty subscribe to all channels. Note that these channel
		* names should be encoded in UTF-8.
		* @return A pointer to an AP_EventSupplier resource, which the caller 
		* is responsible for freeing, using AP_DeleteEventSupplier(). 
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		AP_EventSupplier* (AP_ENGINE_CLIENT_CALL* connectDisconnectableSlowEventConsumer)(AP_EngineManagement* engine, AP_DisconnectableEventConsumer* consumer, const AP_char8* const* channels, AP_bool disconnectIfSlow);

		/**
		* Connect this Engine as an event receiver to another Engine.
		*
		* @param target The Engine to connect to
		* @param channels An array of names representing the
		* channels to subscribe to. This is a null-terminated array
		* of pointers to zero-terminated char arrays. If this is null or empty,
		* subscribe to all channels. Note that these channel names
		* should be encoded in UTF-8.
		* and retry it.
		* @param disconnectSlow disconnect if slow (may be overridden by 
		* correlator configuration or previous connection)
		* @param mode If CONNECT_PARALLEL, connect in parallel
		* @return true if successful
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		AP_bool (AP_ENGINE_CLIENT_CALL* attachAsEventConsumerTo)(AP_EngineManagement* engine, AP_EngineManagement* target, const AP_char8* const* channels, AP_bool disconnectSlow, AP_EngineManagement_ConnectMode mode);

		/**
		* Connect this Engine as an event receiver to another Engine.
		*
		* @param hostname The hostname of the Engine to connect to
		* @param port The port of the Engine to connect to
		* @param channels An array of names representing the
		* channels to subscribe to. This is a null-terminated array
		* of pointers to zero-terminated char arrays. If this is null or empty,
		* subscribe to all channels. Note that these channel names
		* should be encoded in UTF-8.
		* and retry it.
		* @param disconnectSlow disconnect if slow (may be overridden by 
		* correlator configuration or previous connection)
		* @param mode If CONNECT_PARALLEL, connect in parallel
		* @return true if successful
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		AP_bool (AP_ENGINE_CLIENT_CALL* attachAsEventConsumerToHost)(AP_EngineManagement* engine, const AP_char8* hostname, AP_uint16 port, const AP_char8* const* channels, AP_bool disconnectSlow, AP_EngineManagement_ConnectMode mode);

		/**
		* Unsubscribe as an event receiver from another engine.
		*
		* @param target The Engine to unsubscribe from.
		* @param channels An array of names representing the
		* channels to unsubscribe from. This is a null-terminated array
		* of pointers to zero-terminated char arrays. If this is null or empty 
		* unsubscribe from all channels.
		* @param mode If CONNECT_PARALLEL, connect in parallel
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/					
		void (AP_ENGINE_CLIENT_CALL* detachAsEventConsumerFrom)(AP_EngineManagement* engine, AP_EngineManagement* target, const AP_char8* const* channels, AP_EngineManagement_ConnectMode mode);

		/**
		* Unsubscribe as an event receiver from another engine.
		*
		* @param hostname The host of the Engine to unsubscribe from.
		* @param port The port of the Engine to unsubscribe from.
		* @param channels An array of names representing the
		* channels to unsubscribe from. This is a null-terminated array
		* of pointers to zero-terminated char arrays. If this is null or empty 
		* unsubscribe from all channels.
		* @param mode If CONNECT_PARALLEL, connect in parallel
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/					
		void (AP_ENGINE_CLIENT_CALL* detachAsEventConsumerFromHost)(AP_EngineManagement* engine, const AP_char8* hostname, AP_uint16 port, const AP_char8* const* channels, AP_EngineManagement_ConnectMode mode);

		/**
		* Returns information about the monitors, event types and 
		* container types which exist in the engine. Use AP_DeleteEngineInfo
		* to delete the returned object when it is no longer needed.
		*
		* @return Information about the engine.
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		AP_EngineInfo* (AP_ENGINE_CLIENT_CALL* inspectEngine)(AP_EngineManagement* engine);

		/**
		* This method is used to check that the Engine is
		* still alive, potentially reconnecting if needed. 
		* If the Engine is alive it returns
		* normally. If there is a problem then an
		* EngineException is thrown.
		*
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* ping)(AP_EngineManagement* engine);

		/**
		* This method is used to check that this object is
		* still connected to the Engine. It will never
		* reconnect. Returns true if connected.
		* @deprecated The C++ API should be used in preference
		*/
		AP_bool (AP_ENGINE_CLIENT_CALL* isConnected)(AP_EngineManagement* engine);

		/**
		* Inject MonitorScript text into the Engine,
		* returning any warnings produced by the
		* MonitorScript compiler; maintained for compatibility only, 
		* please use injectMonitorScriptWithWarningsFilename instead of 
		* this function. 
		*
		* @param script MonitorScript text to be injected.
		* @return A pointer to a NULL terminated array of
		* warnings which must be deleted with the
		* AP_DeleteWarnings() function, or NULL if there are
		* none
		* @exception EngineException, which may contain
		* warnings as well as an error message
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* const* (AP_ENGINE_CLIENT_CALL* injectMonitorScriptWithWarnings)(AP_EngineManagement* engine, AP_MonitorScript* script);

		/**
		* Inject MonitorScript text into the Engine,
		* supplying the filename it was injected from and
		* returning any warnings produced by the
		* MonitorScript compiler
		*
		* @param script MonitorScript text to be injected.
		* @param filename The filename the MonitorScript text 
		* was injected from
		* @return A pointer to a NULL terminated array of
		* warnings which must be deleted with the
		* deleteWarnings() function, or NULL if there are
		* none
		* @exception EngineException, which may contain
		* warnings as well as an error message
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* const* (AP_ENGINE_CLIENT_CALL* injectMonitorScriptWithWarningsFilename)(AP_EngineManagement* engine, AP_MonitorScript* script, const char *filename);

		/**
		* Injects a Java application (a jar) into the engine; maintained for 
		* compatibility only, please use injectJavaWithWarningsFilename instead of this function. 
		*
		* @param jarbytes A pointer to the array of bytes
		* containing the jar
		* @param size The size of the jar
		* @return A pointer to a NULL terminated array of
		* warnings which must be deleted with the
		* AP_DeleteWarnings() function, or NULL if there are
		* none
		* @exception EngineException, which may contain
		* warnings as well as an error message
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* const* (AP_ENGINE_CLIENT_CALL* injectJavaWithWarnings)(AP_EngineManagement* engine, const AP_uint8* jarbytes, AP_uint32 size);

		/**
		* Injects a Java application (a jar) into the engine.
		*
		* @param jarbytes A pointer to the array of bytes
		* containing the jar
		* @param size The size of the jar
		* @param filename The name of the jar file
		* @return A pointer to a NULL terminated array of
		* warnings which must be deleted with the
		* deleteWarnings() function, or NULL if there are
		* none
		* @exception EngineException, which may contain
		* warnings as well as an error message
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* const* (AP_ENGINE_CLIENT_CALL* injectJavaWithWarningsFilename)(AP_EngineManagement* engine, const AP_uint8* jarbytes, AP_uint32 size, const char *filename);

		/**
		* Injects a CDP (Correlator Deployment Package file) into the engine; maintained for 
		* compatibility only, please use injectCDPWithWarningsFilename instead of this function. 
		*
		* @param cdpbytes A pointer to the array of bytes
		* containing the CDP file
		* @param size The size of the CDP file
		* @return A pointer to a NULL terminated array of
		* warnings which must be deleted with the
		* AP_DeleteWarnings() function, or NULL if there are
		* none
		* @exception EngineException, which may contain
		* warnings as well as an error message
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* const* (AP_ENGINE_CLIENT_CALL* injectCDPWithWarnings)(AP_EngineManagement* engine, const AP_uint8* cdpbytes, AP_uint32 size);

		/**
		* Injects a CDP (Correlator Deployment Package file) into the engine
		*
		* @param cdpbytes A pointer to the array of bytes
		* containing the CDP file
		* @param size The size of the CDP file
		* @param filename The name of the CDP file
		* @return A pointer to a NULL terminated array of
		* warnings which must be deleted with the
		* deleteWarnings() function, or NULL if there are
		* none
		* @exception EngineException, which may contain
		* warnings as well as an error message
		* @deprecated The C++ API should be used in preference
		*/
		const AP_char8* const* (AP_ENGINE_CLIENT_CALL* injectCDPWithWarningsFilename)(AP_EngineManagement* engine, const AP_uint8* cdpbytes, AP_uint32 size, const char *filename);

		/**
		* Inject events into the Engine 
		*
		* @param events An array of pointers to AP_Event instances
		* containing the events to inject into the Engine.
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* sendEventsNoBatching)(AP_EngineManagement* engine, AP_Event** events);

		/**
		* Inject events into the Engine (inherited from
		* EventConsumer), automatically batching messages
		* on a separate thread. May return before events are 
		* sent - call flushEvents before exiting.
		*
		* @param events An array of pointers to AP_Event instances
		* containing the events to inject into the Engine.
		* @exception EngineException
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* sendEvents)(AP_EngineManagement* engine, AP_Event** events);

		/**
		* Send any outstanding events sent via sendEvents.
		* @deprecated The C++ API should be used in preference
		*/
		void (AP_ENGINE_CLIENT_CALL* flushEvents)(AP_EngineManagement* engine);
	};

	/**
	 * EngineManagement
	 * @deprecated The C++ API should be used in preference
	 */
	struct AP_EngineManagement_API
	{
		void *reserved;
		/** Pointer to function API */
		struct AP_EngineManagement_Functions* functions;
		/** Pointer to this object with an EventConsumer facade */
		AP_EventConsumer* s_EventConsumer;
	};

	struct AP_EngineManagement_Stubs
	{
		struct AP_EngineManagement_API api;

#ifdef __cplusplus
		void injectMonitorScript(AP_MonitorScript* script) {
			api.functions->injectMonitorScript(this, script);
		}

		void injectJava(const AP_uint8* jarbytes, AP_uint32 size) {
			api.functions->injectJava(this, jarbytes, size);
		}

		void injectCDP(const AP_uint8* cdpbytes, AP_uint32 size, const char* filename) {
			api.functions->injectCDP(this, cdpbytes, size, filename);
		}

		const AP_char8* const* injectMonitorScriptWithWarnings(AP_MonitorScript* script) {
			return api.functions->injectMonitorScriptWithWarnings(this, script);
		}

		const AP_char8* const* injectMonitorScriptWithWarningsFilename(AP_MonitorScript* script, const char *filename) {
			return api.functions->injectMonitorScriptWithWarningsFilename(this, script, filename);
		}

		const AP_char8* const* injectJavaWithWarnings(const AP_uint8* jarbytes, AP_uint32 size) {
			return api.functions->injectJavaWithWarnings(this, jarbytes, size);
		}

		const AP_char8* const* injectJavaWithWarningsFilename(const AP_uint8* jarbytes, AP_uint32 size, const char *filename) {
			return api.functions->injectJavaWithWarningsFilename(this, jarbytes, size, filename);
		}

		const AP_char8* const* injectCDPWithWarnings(const AP_uint8* cdpbytes, AP_uint32 size) {
			return api.functions->injectCDPWithWarnings(this, cdpbytes, size);
		}

		const AP_char8* const* injectCDPWithWarningsFilename(const AP_uint8* cdpbytes, AP_uint32 size, const char *filename) {
			return api.functions->injectCDPWithWarningsFilename(this, cdpbytes, size, filename);
		}
		void deleteName(const AP_char8* name) {
			api.functions->deleteName(this, name);
		}

		void forceDeleteName(const AP_char8* name) {
			api.functions->forceDeleteName(this, name);
		};

		void killName(const AP_char8* name) {
			api.functions->killName(this, name);
		}

		void deleteAll() {
			api.functions->deleteAll(this);
		}

		AP_EngineStatus* getStatus() {
			return api.functions->getStatus(this);
		}

		AP_EventSupplier* connectEventConsumer(AP_EventConsumer* consumer, const AP_char8* const* channels) {
			return api.functions->connectEventConsumer(this, consumer, channels);
		}

		AP_EventSupplier* connectDisconnectableEventConsumer(AP_DisconnectableEventConsumer* consumer, const AP_char8* const* channels) {
			return api.functions->connectDisconnectableEventConsumer(this, consumer, channels);
		}

		AP_EventSupplier* connectDisconnectableSlowEventConsumer(AP_DisconnectableEventConsumer* consumer, const AP_char8* const* channels, bool disconnectSlow) {
			return api.functions->connectDisconnectableSlowEventConsumer(this, consumer, channels, disconnectSlow);
		}

		AP_bool attachAsEventConsumerTo(AP_EngineManagement* target, const AP_char8* const* channels, AP_bool disconnectSlow, AP_EngineManagement_ConnectMode mode) {
			return api.functions->attachAsEventConsumerTo(this, target, channels, disconnectSlow, mode);
		}

		AP_bool attachAsEventConsumerToHost(const AP_char8* hostname, AP_uint16 port, const AP_char8* const* channels, AP_bool disconnectSlow, AP_EngineManagement_ConnectMode mode) {
			return api.functions->attachAsEventConsumerToHost(this, hostname, port, channels, disconnectSlow, mode);
		}

		void detachAsEventConsumerFrom(AP_EngineManagement* target, const AP_char8* const* channels, AP_EngineManagement_ConnectMode mode) {
			api.functions->detachAsEventConsumerFrom(this, target, channels, mode);
		}

		void detachAsEventConsumerFromHost(const AP_char8* hostname, AP_uint16 port, const AP_char8* const* channels, AP_EngineManagement_ConnectMode mode) {
			api.functions->detachAsEventConsumerFromHost(this, hostname, port, channels, mode);
		}

		AP_EngineInfo* inspectEngine() {
			return api.functions->inspectEngine(this);
		}

		void ping() {
			api.functions->ping(this);
		}

		AP_bool isConnected() {
			return api.functions->isConnected(this);
		}

		void sendEvents(AP_Event** events) {
			api.functions->sendEvents(this, events);
		}
		void sendEventsNoBatching(AP_Event** events) {
			api.functions->sendEventsNoBatching(this, events);
		}
		void flushEvents(AP_Event** events) {
			api.functions->flushEvents(this);
		}
#endif /* __cplusplus */
	};


	/* Static functions from C++ API */

	/**
	* This function must be called once per process first before
	* any other Engine operations are carried out.
	*
	* @exception EngineException
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_EngineInit(const char *processName);

	/**
	* This function (or engineInit) must be called once per process first before
	* any other Engine operations are carried out.
	*
	* @exception EngineException
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_EngineInitNoMessaging(const char *processName);

	/**
	* This function must be called once per process before the
	* application closes down. It ensures that communications are
	* shutdown properly and state cleaned up.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_EngineShutdown();

	/**
	* This function attempts to establish a connection to an
	* Engine. It returns an AP_EngineManagement instance that can be used
	* to access the Engine.
	*
	* @param hostname The machine name where an Engine can be located.
	* @param port The port that the Engine is listening on for
	* transport negotiations with the client.
	* @return A pointer to AP_EngineManagement if an Engine is located and
	* a connection established.
	* @exception EngineException
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API AP_EngineManagement* AP_ENGINE_CLIENT_CALL AP_ConnectToEngine(const AP_char8* hostname, AP_uint16 port);

	/**
	* Attempt to establish a receive-only connection to an Engine listening
	* on the named host and port.  If successful, the returned
	* AP_EngineManagement can be used for the connectEventConsumer(),
	* getStatus(), inspectEngine() and ping() operations.
	* All other operations will fail.
	*
	* @param hostname The hostname of the machine the Engine is running on.
	* @param port The port that the Engine is listening on.
	* @return An EngineManagement object operating in receive-only mode.
	* @exception EngineException If the connection cannot be established.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API AP_EngineManagement* AP_ENGINE_CLIENT_CALL AP_connectToEngineReceiveOnly(const AP_char8* hostname, AP_uint16 port);

	/**
	* Attempt to establish a monitor-only connection to an Engine listening
	* on the named host and port.  If successful, the returned pointer to
	* AP_EngineManagement can be used for the getStatus(), 
	* inspectEngine() and ping() operations.  All other operations will fail.
	*
	* @param hostname The hostname of the machine the Engine is running on.
	* @param port The port that the Engine is listening on.
	* @return An EngineManagement object operating in monitor-only mode.
	* @exception EngineException If the connection cannot be established.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API AP_EngineManagement* AP_ENGINE_CLIENT_CALL AP_connectToEngineMonitorOnly(const AP_char8* hostname, AP_uint16 port);

	/* Move to a private header ? */
	AP_ENGINE_CLIENT_API AP_EngineStatus* AP_ENGINE_CLIENT_CALL engineManagement_getStatusOpaque(void *opaque_EngineManagement, char **errmsg);

	/**
	* This function allows disconnection from an Engine.
	*
	* Any EventSuppliers created from the AP_EngineManagement must be
	* deleted explicitly using AP_DeleteEventSupplier in addition to 
	* disconnecting the AP_EngineManagement object.
	*
	* @param corr	The Engine to disconnect from.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_DisconnectFromEngine(AP_EngineManagement* engine);

	/**
	* This function allows creation of AP_MonitorScript instances.
	*
	* @param monitorString	MonitorScript monitor/event definitions
	* @return A pointer to AP_MonitorScript instance.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API AP_MonitorScript* AP_ENGINE_CLIENT_CALL AP_CreateMonitorScript(const AP_char8* text);

	/**
	* This function allows deletion of AP_MonitorScript instances.
	*
	* @param mon	The pointer to the AP_MonitorScript instance to delete.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_DeleteMonitorScript(AP_MonitorScript* script);

	/**
	* This function allows creation of an AP_Event instance.
	*
	* @param eventString C style string representing the event
	* instance in MonitorScript. Note that this string should
	* be encoded in UTF-8.
	* @return A pointer to an AP_Event instance.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API AP_Event* AP_ENGINE_CLIENT_CALL AP_CreateEvent(const AP_char8* text, const AP_char8* channel);

	/**
	* This function allows deletion of an AP_Event instance.
	*
	* @param event A pointer to an AP_Event instance.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_DeleteEvent(AP_Event* event);

	/**
	* This function disconnects and deletes an AP_EventSupplier instance.
	*
	* It also stops the associated AP_EventConsumer from receiving
	* events. 
	*
	* @param supplier	A pointer to an AP_EventSupplier instance.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_DeleteEventSupplier(AP_EventSupplier* supplier);

	/**
	* This function allows deletion of an AP_EngineStatus instance.
	*
	* @param status	A pointer to an AP_EngineStatus instance.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_DeleteEngineStatus(AP_EngineStatus* status);

	/**
	* This function allows deletion of an AP_EngineInfo instance.
	* All instances returned by any calls to the methods of the 
	* EngineInfo object are also deleted, so after calling
	* inspectEngine, AP_DeleteEngineInfo is the only method which 
	* needs to be called to clean up.
	*
	* @param info	A pointer to an AP_EngineInfo instance.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_DeleteEngineInfo(AP_EngineInfo* info);

	/**
	* This function allows deletion of warnings.
	*
	* @param warnings	A pointer to a NULL terminated list of warnings
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_DeleteWarnings(const AP_char8* const* warnings);
	
	/**
	 * Available logging levels 
	 * @deprecated The C++ API should be used in preference
	 */
	typedef enum {
		/** Nothing logged */
		OFF_LEVEL,
		/** Startup and other important messages */
		FORCE_LEVEL,
		/** Critical messages */
		CRIT_LEVEL,
		/** Fatal messages */
		FATAL_LEVEL,
		/** Error messages */
		ERROR_LEVEL,
		/** Warning messages */
		WARN_LEVEL,
		/** Information messages (default) */
		INFO_LEVEL,
		/** Debugging messages */
		DEBUG_LEVEL,
		/** More-verbose debug messages */
		TRACE_LEVEL
	} AP_EngineClientLogLevel;
	
	/**
	 * Sets the level at which the client library will log information.
	 *
	 * @param level The level to log at
	 * @deprecated The C++ API should be used in preference
	 */
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_SetLogLevel(AP_EngineClientLogLevel level);
	
	/**
	 * Sets the file to which the client library should log information.
	 *
	 * @param filename The filename to which the library should log. Will log
	 * to stderr if the filename cannot be opened.
	 * @param truncate If non-zero the log file will be truncated when it is
	 * opened. If zero then it will simply be appended to.
	 * set.
	 * @deprecated The C++ API should be used in preference
	 */
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_SetLogFile(const AP_char8* filename, int truncate);
	
	/**
	 * Sets the file to which the client library should log information.
	 * 
	 *
	 * @param filename The filename, specified using UTF-8 characters, 
	 * to which the library should log. Will log
	 * to stderr if the filename cannot be opened.
	 * @param truncate If non-zero the log file will be truncated when it is
	 * opened. If zero then it will simply be appended to.
	 * set.
	 * @deprecated The C++ API should be used in preference
	 */
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_SetLogFileUTF8(const AP_char8* filename, int truncate);

	/**
	* Return the FD for the libraries logger.
	* 
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API int AP_ENGINE_CLIENT_CALL AP_GetCoreLogFD();
	
	/**
	 * Sets the mutex which the client library logger should use. For internal use only.
	 * @deprecated The C++ API should be used in preference
	 * 
	 */
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_SetLogMutex(void *);
	
	/**
	 * Re-opens the log file. Also called if SIGHUP is received.
	 * 
	 * @deprecated The C++ API should be used in preference
	 */
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_ReOpenLog();
	
	/* Convert a string in local encoding to UTF-8, as required by most of the Apama API */
	AP_ENGINE_CLIENT_API AP_char8* AP_ENGINE_CLIENT_CALL AP_ConvertToUTF8(const AP_char8* s);

	/* Convert a string in UTF-8 to the local encoding, since most of the Apama API returns UTF-8 */
	AP_ENGINE_CLIENT_API AP_char8* AP_ENGINE_CLIENT_CALL AP_ConvertFromUTF8(const AP_char8* s);


	/* Exception checking/handling functions -- these don't appear in the C++ API */

	typedef enum {
		/** An exception in the client library */
		AP_ENGINE_EXCEPTION
	} AP_EngineClientExceptionType;

	/**
	* Checks if an exeption was thrown and saved by the library
	*
	* @return true if an exception was thrown, false otherwise
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API AP_bool AP_ENGINE_CLIENT_CALL AP_CheckException();

	/**
	* Returns the type of the pending exception
	*
	* @return type of the pending exception
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API AP_EngineClientExceptionType AP_ENGINE_CLIENT_CALL AP_GetExceptionType();

	/**
	* Retrieve the message enclosed within the exception.
	*
	* @return C style string with details of the error that
	* caused the operation to fail.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API const AP_char8*  AP_ENGINE_CLIENT_CALL AP_GetExceptionMessage();

	/**
	* Retrieve the set of warnings associated with the exception
	*
	* @return A pointer to a NULL terminated list of warnings,
	* possibly NULL if there are none. This list is owned by
	* the exception object. It should not be deleted.
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API const AP_char8* const*  AP_ENGINE_CLIENT_CALL AP_GetExceptionWarnings();

	/**
	* Clears the pending exception
	*
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_ClearException();

	/**
	* Set custom parameters for this instance of the client library.
	* Must be called before AP_EngineInit. The parameters are passed 
	* as a set of name, value pairs formatted as
	* <name1>=<value1>;<name2=value2>;
	* (i.e. equals sign between name and value, and semicolon separating 
	* the name value pairs).
	*
	* The following parameters are defined so far.
	*
	* ConfigFile - A filename from which to read a configuration file
	* LogicalID - The logical id with which this component should be
	* identified (a 64-bit integer)
	* @deprecated The C++ API should be used in preference
	*/
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL AP_SetEngineParams(const AP_char8* params);
	
	/**
	 * @internal
	 * Returns non-zero if the given event name is part of the OutOfBand API or zero if it is not.
	 *
	 * This is an internal-only function.
	 * @deprecated The C++ API should be used in preference
	 */
	AP_ENGINE_CLIENT_API int AP_ENGINE_CLIENT_CALL AP_IsOutOfBandEvent(const AP_char8* eventName);

	/**
	 * Free any char* string returned from the client API
	 * @deprecated The C++ API should be used in preference
	 */
	AP_ENGINE_CLIENT_API void AP_ENGINE_CLIENT_CALL  AP_char8free(char *string);

#ifdef __cplusplus
}	/* extern "C" */
#endif /* __cplusplus */


#endif /* ENGINE_CLIENT_C_H */
