// xhr_props.js
// $Copyright(c) 2008, 2009-2011 Progress Software Corporation (PSC). All rights reserved.$
// $Copyright (c) 2013, 2016-2017 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
// Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG

// Small amount of JavaScript for doing XmlHttpResponse, and putting the result in a table

// All the XML is comprised of 3 elements types:
// <map>, <list>, and <prop>
// All of these elements have an attribute called 'name'
// Difference between list and map is simply that the items in a map have unique names.
// Each elements may contain any of the other elements. The prop element may only have one child. 

var xmlhttp = null;
var onceXmlHttp = null;
var targetElementId = null;

function initXHR() {
	var oXmlHttpReq = null;
	if (window.XMLHttpRequest) {
		oXmlHttpReq = new XMLHttpRequest();
		if ( typeof oXmlHttpReq.overrideMimeType != 'undefined') { 
			oXmlHttpReq.overrideMimeType('text/xml'); 
		}
	} else if (window.ActiveXObject) {
		oXmlHttpReq = new ActiveXObject("Microsoft.XMLHTTP");
	} else {
		alert('DEBUG - initXHR() Perhaps your browser does not support xmlhttprequests?');
	}
	return oXmlHttpReq;
}

function normalCallback() {
	//alert("DEBUG - initXHRCallback() - onreadystatechange - xmlhttp.readyState="+xmlhttp.readyState);
	switch (xmlhttp.readyState) {
		case 4: // COMPLETED
			if (xmlhttp.status == 200) {
				// do something with the results
				//alert('DEBUG - initXHRCallback() - onreadystatechange - GOOD');
				displayXML(xmlhttp.responseXML, xmlhttp.responseText, 200, xmlhttp.statusText);
			}
			else {
				//alert("DEBUG - initXHRCallback() - onreadystatechange - xmlhttp.readyState=4 (COMPLETED), xmlhttp.status="+xmlhttp.status);
				if (xmlhttp.responseXML) {
					//alert("DEBUG - initXHRCallback() - got XML");
					displayXML(xmlhttp.responseXML, xmlhttp.responseText, xmlhttp.status, xmlhttp.statusText);
				}
				else {
					displayXML(null, xmlhttp.responseText, xmlhttp.status, xmlhttp.statusText);
					alert("DEBUG - initXHRCallback() - no XML");
					alert("DEBUG - initXHRCallback() - onreadystatechange - xmlhttp.readyState=4 (COMPLETED), xmlhttp.status="+xmlhttp.status+", xmlhttp.statusText="+xmlhttp.statusText);
				}
			}
			break;
		case 3: // INTERACTIVE
			//alert("initXHRCallback() - onreadystatechange - xmlhttp.readyState=3 (INTERACTIVE)");
			break;
		case 2: // LOADED
			//alert("initXHRCallback() - onreadystatechange - xmlhttp.readyState=2 (LOADED)");
			break;
		case 1: // LOADING
			//alert("initXHRCallback() - onreadystatechange - xmlhttp.readyState=1 (LOADING)");
			break;
		case 0: // UNINITIALIZED
		default: // unexpected state - should be one of the above - treat just like 0
			alert("DEBUG - initXHRCallback() - onreadystatechange - UNINITIALIZED - xmlhttp.readyState="+xmlhttp.readyState);
		
	} //switch
}
	


function displayXML(responseXML, responseText, intHttpStatusCode, strHttpStatusText) { 
	var txt = "";
	
	// debug
	txt = txt + "<div class='xhr_status'>";
	txt = txt + "<span class='xhr_status_text'>"
	txt = txt + "HTTP status: "+intHttpStatusCode+" ("+strHttpStatusText+") <br>";
	txt = txt + "Time of last response: " + new Date();
	txt = txt + "</span>"
	txt = txt + "<span class='xhr_status_response'>"
	//alert("DEBUG - "+txt);
	
	if (200 == intHttpStatusCode) {
		txt = txt + '<form><input type="button" id="get_data_id" value="Refresh Data" onclick="reloadDataLocal();"/></form>';
	}
	else {
		if (400 == intHttpStatusCode) {
			txt = txt + "<h4>Bad client request (HTTP code 400)</h4>";
		}
		else if (404 == intHttpStatusCode) {
			txt = txt + "<h4>Resource not found (HTTP code 404)</h4>";
		}
		else if (500 == intHttpStatusCode) {
			txt = txt + "<h4>Server error (HTTP code 500)</h4>";
		}
		else {
			txt = txt + "<h4>Unexpected server response (HTTP code "+intHttpStatusCode+")</h4>";
		}
	}
	txt = txt + "</span>"
	txt = txt + "</div>";
	var doc = null;
	if (responseXML) {
		doc = responseXML.documentElement;  // this will be  <map name="apama-response">, or similar
	}
	if (doc) {
		//alert( "DEBUG - doc.nodeName: '"+doc.nodeName+"', doc.childNodes.length: "+doc.childNodes.length );
		var rootNodeList = doc.childNodes;

		txt = txt + iterateChildren(rootNodeList,true);
	}
	else {
		if (responseText) {
			txt = txt + "<pre>"+responseText+"</pre>";
		}
		//alert("DEBUG - no documentElement property (or null) in the responseXML.");		
	}
	
	document.getElementById(targetElementId).innerHTML=txt; 
}			


// Iterate all the children in a container (map or list)
function iterateChildren(nodeList, boolRootTable) {
	var resultTxt="";
	if (boolRootTable) {
		resultTxt = "<table class='tableApamaResponse'><tr><th/><th>Name</th><th>Value</th></tr>";
	}
	else {
		resultTxt = "<table width='100%' class='tableApamaResponse'><tr><th/><th>Name</th><th>Value</th></tr>";
	}
	var boolAlt=false;
	var intRow=1
	var item;
	var NODE_ELEMENT=1;
	for (var i=0; i<nodeList.length; i++) {
		item = nodeList.item(i);
		if (item.nodeType==NODE_ELEMENT) {
			if ("prop"==item.nodeName) {
				resultTxt = resultTxt + buildHTMLForPropertyNode(item, boolAlt, intRow);
				boolAlt = !boolAlt;
				intRow = intRow + 1;
			}
			else if ("list"==item.nodeName) {
				listTxt = buildHTMLForListNode(item, boolAlt, intRow);
				resultTxt = resultTxt + listTxt;
				boolAlt = !boolAlt;
				intRow = intRow + 1;
			}
			else if ("map"==item.nodeName) {
				listTxt = buildHTMLForListNode(item, boolAlt, intRow);
				resultTxt = resultTxt + listTxt;
				boolAlt = !boolAlt;
				intRow = intRow + 1;
			}

		}
	}
	resultTxt=resultTxt + "</table>"; 
	return resultTxt;
}



function escapeHtml(unsafe) {
	return unsafe
		.replace(/&/g, "&amp;")
		.replace(/</g, "&lt;")
		.replace(/>/g, "&gt;")
		.replace(/"/g, "&quot;")
		.replace(/'/g, "&#039;");
}



// Build the HTML for a single <prop> element
function buildHTMLForPropertyNode(item, boolAlt, intRow) {
	var resultTxt = "";
	var altTxt = "norm";  if (boolAlt) {altTxt="alt";}
	resultTxt=resultTxt + "<tr valign='top'  class='"+altTxt+"'><td>"+intRow+"</td>"; 

	// name
	try { 
		resultTxt=resultTxt + "<td>" + escapeHtml(item.getAttribute("name")) + "</td>";
	} 
	catch (er) { 
		resultTxt=resultTxt + "<td> </td>"; 
	} 
	
	// value
	try { 
		resultTxt=resultTxt + "<td class='value'>";
		if (window.location.pathname == "/info/categories") {
			resultTxt += "<a href='/info/"+item.firstChild.nodeValue+"'>"+item.firstChild.nodeValue+"</a>";
		}
		else {
			resultTxt += escapeHtml(item.firstChild.nodeValue);
		}
		resultTxt += "</td>";
	} 
	catch (er) { 
		resultTxt=resultTxt + "<td> </td>"; 
	}
	resultTxt=resultTxt + "</tr>"; 
		
	return resultTxt;
}

// Build the HTML for a single <list> element
function buildHTMLForListNode(item, boolAlt, intRow) {
	var resultTxt = "";
	var altTxt = "norm";  if (boolAlt) {altTxt="alt";}
	resultTxt=resultTxt + "<tr valign='top'  class='"+altTxt+"'><td>"+intRow+"</td>"; 

	// name
	try { 
		resultTxt=resultTxt + "<td>" + item.getAttribute("name") + "</td>";
	} 
	catch (er) { 
		resultTxt=resultTxt + "<td> </td>"; 
	} 
	
	// value
	if (item.firstChild) {
		if (item.childNodes.length > 0) {
			//resultTxt=resultTxt + "<td class='value'>" + item.childNodes.length + " items<br>";
			resultTxt=resultTxt + "<td class='nested'>";
			resultTxt=resultTxt + iterateChildren(item.childNodes, false);
			resultTxt=resultTxt + "</td>";
		}
		else {	
			resultTxt=resultTxt + "<td class='value'>0 items</td>";
		}
	}
	else {
		resultTxt=resultTxt + "<td class='value'>0 items</td>";
	}

	resultTxt=resultTxt + "</tr>"; 
		
	return resultTxt;
}


function doXHR_get(oXmlHttpReq, uri, callbackFunction) {
	doXHR(oXmlHttpReq, 'GET', uri, true, null, callbackFunction );
}

// doHXHR()
// verb - one of GET, POST, HEAD, etc
// uri - the endpoint
// async - normally true
// data - what to send - maybe null
function doXHR(oXmlHttpReq, verb, uri, async, data, callbackFunction) {
	oXmlHttpReq.open(verb, uri, async);
	oXmlHttpReq.onreadystatechange = callbackFunction;
	//alert("doXHR() uri="+uri);
	oXmlHttpReq.send(data);
}

function getData(uri, htmlId) {
	//alert("DEBUG - About to request more data. uri: '"+uri+"', target HTML element Id: '"+htmlId+"'");
	targetElementId = htmlId;
	doXHR_get(xmlhttp, uri,normalCallback);
}

function licenceCallback() {
	var item = null;
	var NODE_ELEMENT=1;
	if (4==onceXmlHttp.readyState) {
		if (onceXmlHttp.status == 200) {
			//alert('DEBUG - simpleNameCallback() - onreadystatechange - GOOD');
			if (onceXmlHttp.responseXML) {
				//alert("DEBUG - simpleNameCallback() - got XML");
				//displayXML(onceXmlHttp.responseXML, onceXmlHttp.responseText, onceXmlHttp.status, onceXmlHttp.statusText);
				
				doc = onceXmlHttp.responseXML.documentElement;  // this will be  <map name="apama-response">, or similar
				rootNodeList = doc.childNodes;
				for (var i=0; i<rootNodeList.length; i++) {
					item = rootNodeList.item(i);
					if (item.nodeType==NODE_ELEMENT) {
						if ("prop"==item.nodeName) {
							if ("licenseStatus"==item.getAttribute("name")) {
								licenceStatus = item.firstChild.nodeValue;
								if ("noLicenseFile" == licenceStatus) {
									document.getElementById("banner").appendChild(document.createTextNode(" - No license file")); 
								}
							}
						}
					}
				}
			}
			else {
				alert("DEBUG - licenceCallback() - no XML");
				alert("DEBUG - licenceCallback() - onreadystatechange - xmlHttp.readyState=4 (COMPLETED), xmlHttp.status="+onceXmlHttp.status+", xmlHttp.statusText="+onceXmlHttp.statusText);
			}
		}
	}	
}
function simpleNameCallback() {
	var componentName = "";
	var componentType = "";
	var componentVersion = "";
	var item = null;
	var NODE_ELEMENT=1;
	if (4==onceXmlHttp.readyState) {
		if (onceXmlHttp.status == 200) {
			//alert('DEBUG - simpleNameCallback() - onreadystatechange - GOOD');
			if (onceXmlHttp.responseXML) {
				//alert("DEBUG - simpleNameCallback() - got XML");
				//displayXML(onceXmlHttp.responseXML, onceXmlHttp.responseText, onceXmlHttp.status, onceXmlHttp.statusText);
				
				doc = onceXmlHttp.responseXML.documentElement;  // this will be  <map name="apama-response">, or similar
				rootNodeList = doc.childNodes;
				for (var i=0; i<rootNodeList.length; i++) {
					item = rootNodeList.item(i);
					if (item.nodeType==NODE_ELEMENT) {
						if ("prop"==item.nodeName) {
							if ("name"==item.getAttribute("name")) {
								componentName = item.firstChild.nodeValue;
							}
							else if ("type"==item.getAttribute("name")) {
								componentType = item.firstChild.nodeValue;
							}
							else if ("componentVersion"==item.getAttribute("name")) {
								componentVersion = item.firstChild.nodeValue;
							}
						}
					}
				}
			}
			else {
				alert("DEBUG - simpleNameCallback() - no XML");
				alert("DEBUG - simpleNameCallback() - onreadystatechange - xmlHttp.readyState=4 (COMPLETED), xmlHttp.status="+onceXmlHttp.status+", xmlHttp.statusText="+onceXmlHttp.statusText);
			}
			var h1 = document.createElement("h1");
			h1.setAttribute("id", "banner");
			h1.appendChild(document.createTextNode(capitalizeComponentType(componentType)+" v"+componentVersion+" ("+componentName+")"));
			var links = document.createElement("ul");
			var llink = document.createElement("li");
			var link = document.createElement("a");
			link.setAttribute("href", "/info");
			link.appendChild(document.createTextNode("Info"));
			llink.appendChild(link);
			links.appendChild(llink);
			llink = document.createElement("li");
			link = document.createElement("a");
			link.setAttribute("href", "/info/argv");
			link.appendChild(document.createTextNode("Args"));
			llink.appendChild(link);
			links.appendChild(llink);
			llink = document.createElement("li");
			link = document.createElement("a");
			link.setAttribute("href", "/info/envp");
			link.appendChild(document.createTextNode("Environment"));
			llink.appendChild(link);
			links.appendChild(llink);
			llink = document.createElement("li");
			link = document.createElement("a");
			link.setAttribute("href", "/connections");
			link.appendChild(document.createTextNode("Connections"));
			llink.appendChild(link);
			links.appendChild(llink);

			if ("correlator"==componentType) {
				llink = document.createElement("li");
				link = document.createElement("a");
				link.setAttribute("href", "/info/license");
				link.appendChild(document.createTextNode("License"));
				llink.appendChild(link);
				links.appendChild(llink);
				llink = document.createElement("li");
				link = document.createElement("a");
				link.setAttribute("href", "/correlator/info");
				link.appendChild(document.createTextNode("Inspect"));
				llink.appendChild(link);
				links.appendChild(llink);
				llink = document.createElement("li");
				link = document.createElement("a");
				link.setAttribute("href", "/correlator/status");
				link.appendChild(document.createTextNode("Watch"));
				llink.appendChild(link);
				links.appendChild(llink);
				llink = document.createElement("li");
				link = document.createElement("a");
				link.setAttribute("href", "/correlator/debug");
				link.appendChild(document.createTextNode("Debug"));
				llink.appendChild(link);
				links.appendChild(llink);
				llink = document.createElement("li");
				link = document.createElement("a");
				link.setAttribute("href", "/correlator/profile");
				link.appendChild(document.createTextNode("Profile"));
				llink.appendChild(link);
				links.appendChild(llink);
				llink = document.createElement("li");
				link = document.createElement("a");
				link.setAttribute("href", "/correlator/applogging");
				link.appendChild(document.createTextNode("Logging"));
				llink.appendChild(link);
				links.appendChild(llink);
				llink = document.createElement("li");
				link = document.createElement("a");
				link.setAttribute("href", "/correlator/types");
				link.appendChild(document.createTextNode("Types"));
				llink.appendChild(link);
				links.appendChild(llink);
				uri = document.location.protocol +"//"+ document.location.host + "/info/license";
				doXHR_get(onceXmlHttp, uri, licenceCallback);
			}
			else if ("iaf"==componentType) {
				llink = document.createElement("li");
				link = document.createElement("a");
				link.setAttribute("href", "/iaf/status");
				link.appendChild(document.createTextNode("Watch"));
				llink.appendChild(link);
				links.appendChild(llink);
				llink = document.createElement("li");
				link = document.createElement("a");
				link.setAttribute("href", "/iaf/manage");
				link.appendChild(document.createTextNode("IAF management"));
				llink.appendChild(link);
				links.appendChild(llink);
			}
			else if ("dashboardDataServer"==componentType || "dashboardDisplayServer"==componentType) {
				llink = document.createElement("li");
				link = document.createElement("a");
				link.setAttribute("href", "/info/sysprop");
				link.appendChild(document.createTextNode("Sysprops"));
				llink.appendChild(link);
				links.appendChild(llink);
				llink = document.createElement("li");
				
				link = document.createElement("a");
				link.setAttribute("href", "/info/stats");
				link.appendChild(document.createTextNode("Stats"));
				llink.appendChild(link);
				links.appendChild(llink);
			}
			llink = document.createElement("li");
			link = document.createElement("a");
			link.setAttribute("href", "/info/categories");
			link.appendChild(document.createTextNode("Categories"));
			llink.appendChild(link);
			links.appendChild(llink);

			document.getElementById("componentName").appendChild(h1); 
			document.getElementById("componentName").appendChild(links); 
			// Set up the active class in the navigation.
			setNaviationActiveClass(links, componentType, window.location.pathname);
		}
		else {
			alert("DEBUG - simpleNameCallback() - onreadystatechange - xmlHttp.readyState=4 (COMPLETED), xmlHttp.status="+onceXmlHttp.status);
		}
	} 
	else {
		//alert("DEBUG - simpleNameCallback() - onreadystatechange - xmlhttp.readyState="+onceXmlHttp.readyState);
	}
}

function capitalizeComponentType(componentType) {
	if ("iaf"==componentType) {
		// iaf is referred as complete capitals
		return "IAF";
	} else if ("dashboardDataServer"==componentType) {
		// spaces introduced
		return "Dashboard Data Server";
	} else if( "dashboardDisplayServer"==componentType) {
		// spaces introduced
		return "Dashboard Display Server";
	}
	return componentType.charAt(0).toUpperCase() + componentType.slice(1);
}

function setNaviationActiveClass(links, componentType, activeHRef) {
	var backUpDebug = null;
	var setActive = false;
	var a_elements = links.getElementsByTagName("a");
	for (var i = 0, len = a_elements.length; i < len; i++ ) {
		// Remove any active class that is set up.
		a_elements[i].className = '';
		var elementHRef = a_elements[i].getAttribute('href')
		// For the stats there is no specific page and also the
		// origin is from the Categories page so we highlight that page.
		// Looks hacky but really no easy way to get here :(.
		// Dashboards have the info/stats page so add it only for correlator and IAF
		//alert (componentType + " : " + activeHRef + " : " + elementHRef)
		if (activeHRef == '/info/stats' && ("iaf"==componentType || "correlator"==componentType) && elementHRef == '/info/categories') {
			a_elements[i].className = 'active';
			setActive = true;
		} else {
			if ("correlator"==componentType) {
				if(elementHRef == '/correlator/debug') {
					backUpDebug = a_elements[i];
				}
			}
			if(elementHRef == activeHRef) {
				// Add the active class.
				a_elements[i].className = 'active';
				setActive = true;
			}
		}
    }
	
	if (setActive == false && backUpDebug != null) {
		backUpDebug.className = 'active';
	}
		
}
function getComponentInfo(objBaseLocation) {
	//alert("DEBUG - getComponentInfo(baseURI='"+objBaseLocation.href+"')");
	uri = objBaseLocation.protocol +"//"+ objBaseLocation.host + "/info";
	//alert("DEBUG - getComponentInfo(constructed URI='"+uri+"')");
	doXHR_get(onceXmlHttp, uri, simpleNameCallback);
}

function init() {
	xmlhttp = initXHR();
	
	onceXmlHttp = initXHR();
}


init();
